import asyncio
import datetime
from typing import (
    Any,
    BinaryIO,
    ClassVar,
    Coroutine,
    Dict,
    Iterable,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)
from typing_extensions import Literal

import aiohttp

from .abc import User as _ABCUser
from .asset import Asset
from .channel import TextChannel
from .embeds import Embed
from .enums import WebhookType
from .file import File
from .guild import Guild
from .http import _WebhookDict
from .mentions import AllowedMentions
from .message import Message
from .mixins import Hashable
from .state import ConnectionState

class WebhookAdapter:
    BASE: ClassVar[str] = ...

    webhook: Webhook
    def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
    ) -> Any: ...
    def delete_webhook(self, *, reason: Optional[str] = ...) -> Any: ...
    def edit_webhook(self, *, reason: Optional[str] = ..., **payload: Any) -> Any: ...
    def handle_execution_response(self, data: Any, *, wait: bool) -> Any: ...
    def execute_webhook(
        self,
        *,
        payload: Dict[str, Any],
        wait: bool = ...,
        file: Optional[Tuple[str, BinaryIO, str]] = ...,
        files: Optional[List[Tuple[str, BinaryIO, str]]] = ...,
    ) -> Any: ...

class AsyncWebhookAdapter(WebhookAdapter):
    session: aiohttp.ClientSession
    loop: asyncio.AbstractEventLoop
    def __init__(self, session: aiohttp.ClientSession) -> None: ...
    async def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
        *,
        files: Optional[Iterable[File]] = ...,
        reason: Optional[str] = ...,
    ) -> Any: ...
    async def handle_execution_response(
        self, response: Any, *, wait: bool
    ) -> Message: ...

class RequestsWebhookAdapter(WebhookAdapter):
    session: Any
    def __init__(self, session: Optional[Any] = ..., *, sleep: bool = ...) -> None: ...
    def request(
        self,
        verb: str,
        url: str,
        payload: Optional[Dict[str, Any]] = ...,
        multipart: Optional[Dict[str, Any]] = ...,
        *,
        files: Optional[Iterable[File]] = ...,
        reason: Optional[str] = ...,
    ) -> Any: ...
    def handle_execution_response(self, response: Any, *, wait: bool) -> Message: ...

_T = TypeVar('_T', bound=Webhook)

class Webhook(Hashable):
    id: int
    type: WebhookType
    token: Optional[str]
    channel_id: Optional[int]
    guild_id: Optional[int]
    name: Optional[str]
    avatar: Optional[str]
    user: Optional[_ABCUser]
    @property
    def url(self) -> str: ...
    @classmethod
    def partial(
        cls: Type[_T], id: int, token: str, *, adapter: WebhookAdapter
    ) -> _T: ...
    @classmethod
    def from_url(cls: Type[_T], url: str, *, adapter: WebhookAdapter) -> _T: ...
    # NOTE: While this method is public, it should never be invoked by users.
    @classmethod
    def from_state(cls: Type[_T], data: _WebhookDict, state: ConnectionState) -> _T: ...
    @property
    def guild(self) -> Optional[Guild]: ...
    @property
    def channel(self) -> Optional[TextChannel]: ...
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def avatar_url(self) -> Asset: ...
    def avatar_url_as(
        self, *, format: Optional[Literal['png', 'jpg', 'jpeg']] = ..., size: int = ...
    ) -> Asset: ...
    def delete(self, *, reason: Optional[str] = ...) -> Coroutine[Any, Any, None]: ...
    def edit(
        self, *, reason: Optional[str] = ..., **kwargs: Any
    ) -> Union[_WebhookDict, Coroutine[Any, Any, _WebhookDict]]: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> Union[Message, Coroutine[Any, Any, Message]]: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[List[File]] = ...,
        embed: Optional[Embed] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> Union[Message, Coroutine[Any, Any, Message]]: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        file: Optional[File] = ...,
        embeds: Optional[List[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> Union[Message, Coroutine[Any, Any, Message]]: ...
    @overload
    def send(
        self,
        content: Optional[str] = ...,
        *,
        wait: bool = ...,
        username: Optional[str] = ...,
        avatar_url: Optional[Union[str, Asset]] = ...,
        tts: bool = ...,
        files: Optional[List[File]] = ...,
        embeds: Optional[List[Embed]] = ...,
        allowed_mentions: Optional[AllowedMentions] = ...,
    ) -> Union[Message, Coroutine[Any, Any, Message]]: ...
    def execute(self, *args: Any, **kwargs: Any) -> Any: ...



import datetime
from typing import Any, List, Optional, Tuple, Union

import discord.abc

from .activity import BaseActivity, Spotify
from .asset import Asset
from .channel import DMChannel, GroupChannel, VoiceChannel
from .colour import Colour
from .enums import DefaultAvatar, Status
from .flags import PublicUserFlags
from .guild import Guild
from .message import Message
from .permissions import Permissions
from .relationship import Relationship
from .role import Role
from .user import Profile, User

class VoiceState:
    deaf: bool
    mute: bool
    self_mute: bool
    self_deaf: bool
    self_stream: bool
    self_video: bool
    afk: bool
    channel: Optional[Union[GroupChannel, VoiceChannel]]
    session_id: str

class Member(discord.abc.Messageable, discord.abc.User):
    joined_at: Optional[datetime.datetime]
    activities: Tuple[Union[BaseActivity, Spotify], ...]
    nick: Optional[str]
    premium_since: Optional[datetime.datetime]
    guild: Guild

    # From discord.user.BaseUser
    name: str
    id: int
    discriminator: str
    avatar: Optional[str]
    bot: bool
    system: bool
    @property
    def public_flags(self) -> PublicUserFlags: ...
    @property
    def avatar_url(self) -> Asset: ...
    def is_avatar_animated(self) -> bool: ...
    def avatar_url_as(
        self, *, format: Optional[str] = ..., static_format: str = ..., size: int = ...
    ) -> Asset: ...
    @property
    def default_avatar(self) -> DefaultAvatar: ...
    @property
    def default_avatar_url(self) -> Asset: ...
    # End from discord.user.BaseUser

    # From discord.user.User
    @property
    def created_at(self) -> datetime.datetime: ...
    @property
    def dm_channel(self) -> Optional[DMChannel]: ...
    async def create_dm(self) -> DMChannel: ...
    @property
    def relationship(self) -> Optional[Relationship]: ...
    async def mutual_friends(self) -> List[User]: ...
    def is_friend(self) -> bool: ...
    def is_blocked(self) -> bool: ...
    async def block(self) -> None: ...
    async def unblock(self) -> None: ...
    async def remove_friend(self) -> None: ...
    async def send_friend_request(self) -> None: ...
    async def profile(self) -> Profile: ...
    # End from discord.user.User
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def status(self) -> Union[Status, str]: ...
    @property
    def raw_status(self) -> str: ...
    @property
    def mobile_status(self) -> Status: ...
    @property
    def desktop_status(self) -> Status: ...
    @property
    def web_status(self) -> Status: ...
    def is_on_mobile(self) -> bool: ...
    @property
    def colour(self) -> Colour: ...
    @property
    def color(self) -> Colour: ...
    @property
    def roles(self) -> List[Role]: ...
    @property
    def mention(self) -> str: ...
    @property
    def display_name(self) -> str: ...
    @property
    def activity(self) -> Union[BaseActivity, Spotify]: ...
    def mentioned_in(self, message: Message) -> bool: ...
    def permissions_in(self, channel: discord.abc.GuildChannel) -> Permissions: ...
    @property
    def top_role(self) -> Role: ...
    @property
    def guild_permissions(self) -> Permissions: ...
    @property
    def voice(self) -> Optional[VoiceState]: ...
    async def ban(
        self, *, reason: Optional[str] = ..., delete_message_days: int = ...
    ) -> None: ...
    async def unban(self, *, reason: Optional[str] = ...) -> None: ...
    async def kick(self, *, reason: Optional[str] = ...) -> None: ...
    async def edit(
        self,
        *,
        reason: Optional[str] = ...,
        nick: Optional[str] = ...,
        mute: bool = ...,
        deafen: bool = ...,
        roles: List[Role] = ...,
        voice_channel: Optional[VoiceChannel] = ...,
    ) -> None: ...
    async def move_to(
        self, channel: Optional[VoiceChannel], *, reason: Optional[str] = ...
    ) -> None: ...
    async def add_roles(
        self,
        *roles: discord.abc.Snowflake,
        reason: Optional[str] = ...,
        atomic: bool = ...,
    ) -> None: ...
    async def remove_roles(
        self,
        *roles: discord.abc.Snowflake,
        reason: Optional[str] = ...,
        atomic: bool = ...,
    ) -> None: ...
